    // Maintain a queue of objects. This class provides a more limited
    // set of capabilities than java.util.LinkedList, for instance.
    // Use an inner class, Node, to provide the internal structure
    // of the queue.
public class Queue {
        // Add an object to the tail of the queue.
    public void add(Object o){
        Node n = new Node(o);
        Node tail = getTail();
        if(tail != null){
            tail.setNext(n);
        }
        else{
            // Whole queue is empty.
            setHead(n);
        }
        setTail(n);
    }

        // Return (and remove) the object at the head of the queue.
        // Return null if the queue is empty.
    public Object next(){
        Object item = null;
        Node head = getHead();
        if(head != null){
            item = head.getItem();
            // Drop the node from the head.
            Node nextNode = head.getNext();
            setHead(nextNode);
            if(nextNode == null){
                // Empty queue.
                setTail(nextNode);
            }
        }
        return item;
    }

        // Return a reference to the object at the head of the queue,
        // without removing it.
        // Return null if the queue is empty.
    public Object peek(){
        Object item = null;
        Node head = getHead();
        if(head != null){
            item = head.getItem();
        }
        return item;
    }

        // Return the length of the queue.
    public int size(){
        int length = 0;
        Node node = getHead();
        while(node != null){
            length++;
            node = node.getNext();
        }
        return length;
    }
        
        // Maintain a reference to a single item in the queue, and
        // a reference to the next in line.
    protected class Node {
        public Node(Object item){
            this.item = item;
        }

        public Object getItem(){
            return item;
        }

        public Node getNext(){
            return next;
        }

        public void setNext(Node n){
            next = n;
        }

        private final Object item;
        private Node next = null;
    }

    protected Node getHead(){
        return head;
    }

    protected void setHead(Node n){
        head = n;
    }

    protected Node getTail(){
        return tail;
    }

    protected void setTail(Node n){
        tail = n;
    }

    // References to the node at the head and tail of the queue.
    // Both references are null when the queue is empty.
    private Node head = null, tail = null;
}
